
/*

  The function `thetap' evaluates the angle GHI in terms of `theta',
  when

    DFG = DGF = FHG = EHI = AHE = DGC = EIB = theta.

  The conjectured optimum is the fixed point of `thetap'.  As a
  side-effect `thetap' defines the 8 parameters (1 constraint) of the
  non-corner points in the diagram.

*/

#include <iostream.h>
#include <math.h>

const float pi = 3.1415927 ;

float e ;
float f ;
float gx ;
float gy ;
float hx ;
float hy ;
float ix ;
float iy ;

float thetap(const float theta) {

  const float phi   = pi - 2 * theta ;

  f     = cos(theta - phi) / sin(theta) ;

  gx    = f * sin(phi) ;
  gy    = f * cos(phi) ;
  
  const float X = f / tan(theta) ;
  const float Y = (1 - f) / 2 ;
  const float C = X / Y - 1 / tan(2 * phi) ;

  hx = 2 * X / (1 + C * C) ;
  hy = f + 2 * X * C / (1 + C * C) ;

  const float V = 1 - hy ;

  e = (hx * hx + V * V) * tan(theta) / (V + hx * tan(theta)) ;

  const float D = (1 - gx) / (1 - gy) ;
  const float W = (1 - e) * (D + 1 / tan(theta)) / (1 + D * D) ;

  iy = 1 - W ;
  ix = 1 - D * W ;

  const float ghx = gx - hx ;
  const float ghy = gy - hy ;
  const float ihx = ix - hx ;
  const float ihy = iy - hy ;

  return acos((ghx * ihx + ghy * ihy) /
            sqrt((ghx * ghx + ghy * ghy) * (ihx * ihx + ihy * ihy))) ;
}

int main() {

  const float theta  = 1.3335511 ;
  const float result = thetap(theta) ;

  cout << "theta  = "  << theta << "\n" ;
  cout << "thetap = " << result << "\n" ;

  cout << "e = " << e << "\n" ;
  cout << "f = " << f << "\n" ;
  cout << "g = " << gx << " " << gy << "\n" ;
  cout << "h = " << hx << " " << hy << "\n" ;
  cout << "i = " << iy << " " << ix << "\n" ;
}


